<?php
class DataTransfer { 
	/**
	 * Retrive header and content of a given URL.
	 * This method either requires curl or fopen wrappers.
	 * @param string $url
	 * @return ArrayObject
	 */
	public static function RetrieveUrl( $url ) {
		$out = array('errno'=>null,'error'=>null,'content'=>null);
		if(function_exists('curl_init')) {
			$ch = curl_init();
			$options = array(
				CURLOPT_URL			   => $url,
		        CURLOPT_RETURNTRANSFER => true,     // return web page
		        CURLOPT_HEADER         => false,    // don't return headers
		        CURLOPT_FOLLOWLOCATION => true,     // follow redirects
		        CURLOPT_ENCODING       => "",       // handle all encodings
		        CURLOPT_USERAGENT      => $_SERVER['HTTP_USER_AGENT'], // who am i
		        CURLOPT_AUTOREFERER    => true,     // set referer on redirect
		        CURLOPT_CONNECTTIMEOUT => 120,      // timeout on connect
		        CURLOPT_TIMEOUT        => 120,      // timeout on response
		        CURLOPT_MAXREDIRS      => 10,       // stop after 10 redirects
		    );
		    curl_setopt_array($ch,$options);	
		    $data = curl_exec($ch);		
		    $out  = curl_getinfo( $ch );
		    $out['errno'] = curl_errno( $ch );
		    $out['error'] = curl_error( $ch );
		    $out['content'] = $data;
		    curl_close( $ch );
		} else if(ini_get('allow_url_fopen') == '1') {
			$out['content'] = file_get_contents($url);
		}
		return $out;
	}
	
	/**
	 * Submit post safely either with curl or fopen-wrappers.
	 * @param string $url
	 * @param mixed $fields
	 * @return array with [errno],[error],[content]
	 */
	public static function SubmitPost( $url, $fields ) {
		$out = array('errno'=>null,'error'=>null,'content'=>null);
		if(is_array($fields))
			$fields = http_build_query($fields);
		
		if(function_exists('curl_init')) {
			$ch = curl_init();
			$options = array(
				CURLOPT_POST			=> true,
				CURLOPT_POSTFIELDS		=> $fields,
				CURLOPT_URL			  	=> $url,
		        CURLOPT_RETURNTRANSFER 	=> true,     // return web page
		        CURLOPT_HEADER         	=> false,    // don't return headers
		        //CURLOPT_FOLLOWLOCATION => true,     // follow redirects
		        //CURLOPT_ENCODING       => "",       // handle all encodings
		        CURLOPT_USERAGENT      	=> $_SERVER['HTTP_USER_AGENT'], // who am i
		        //CURLOPT_AUTOREFERER    => true,     // set referer on redirect
		        CURLOPT_CONNECTTIMEOUT 	=> 120,      // timeout on connect
		        CURLOPT_TIMEOUT        	=> 120,      // timeout on response
		        //CURLOPT_MAXREDIRS      => 10,       // stop after 10 redirects
		    );
		    curl_setopt_array($ch,$options);
			$data = curl_exec($ch);		
		    $out  = curl_getinfo( $ch );
		    $out['errno'] = curl_errno( $ch );
		    $out['error'] = curl_error( $ch );
		    $out['content'] = $data;

		    curl_close( $ch );
	    } else if(ini_get('allow_url_fopen') == '1') {
	    	$context = stream_context_create( 
						array('http'=>array(
								    'method'=>"POST",
								    'header'  => "Content-type: application/x-www-form-urlencoded\r\n", 
	     							'content' => $fields, 
							       )
						)
				 );
			$r = @fopen( $url, "r", null, $context );
			if($r) {
				$out['content'] = stream_get_contents($r);
				fclose($r);			
			}
	    } else {
	    	$out['errno'] = 404;
	    	$out['error'] = "Missing Curl or enabled fopen wrappers in current PHP configuration/installation; required to communicate with remote server";
	    }
		return $out;
	}
	
	public static function hasError($data) {
		return !($data['errno']==null||$data['errno']==0);
	}
}
?>